#include <vector>
#include <stdio.h>
#include <math.h>
#include <iostream>

#include "content.h"
#include "guiApp.h"
#include "../debug.h"
#include "../types.h"

bool RecursiveEnvSearch (const PathParam& params, const std::size_t cur_dist, const vertex_t node, 
	const std::set<vertex_t>& path, std::set<vertex_t>& visited, std::vector<vertex_t>& res_list, const Graph& g) {

	std::size_t close_dist = params.close_dist;
	std::size_t far_dist = params.far_dist;
	float min_prob = params.min_prob;
	
	visited.insert(node);
	bool connected = false;
	if ((cur_dist < far_dist) || (cur_dist <= close_dist)) {
	for (Graph::neighbor_it n_iter = g.neighbors_begin(node); n_iter != g.neighbors_end(node); n_iter++) {
		if (visited.find(n_iter->neighbor) != visited.end()) continue;
		if (exp(-g.edge_weight(node, n_iter->neighbor)) < min_prob) continue;
		if ((cur_dist<= close_dist) && (path.find(n_iter->neighbor) != path.end())) { 
			connected = true;
			continue;
		}
		if (RecursiveEnvSearch(params, cur_dist + 1, n_iter->neighbor, path, visited, res_list, g)) connected = true;
	}
	}
	
	if ((cur_dist <= far_dist) || (connected == true)) res_list.push_back(node);
	return connected;
}

std::vector<vertex_t> GetEnvironment(const std::vector<vertex_t>& vertices, const PathParam& params, const Graph& g) {
	typedef std::vector<vertex_t>::const_iterator vertex_vec_it;	

	std::set<vertex_t> visited;
	std::set<vertex_t> path;
	std::vector<vertex_t> result_vec;
	std::vector<vertex_t> result_vec_total;

	float min_prob = params.min_prob;

	for (vertex_vec_it v_iter = vertices.begin(); v_iter != vertices.end(); v_iter++)
		path.insert(*v_iter);

	for (vertex_vec_it v_iter = vertices.begin(); v_iter != vertices.end(); v_iter++) {
		result_vec_total.push_back(*v_iter);
		visited.insert(*v_iter);

		for (Graph::neighbor_it n_iter = g.neighbors_begin(*v_iter); n_iter != g.neighbors_end(*v_iter); n_iter++) {
			if (exp(-g.edge_weight(*v_iter, n_iter->neighbor)) < min_prob) continue;
			if (path.find(n_iter->neighbor) != path.end()) continue;
			RecursiveEnvSearch(params, 1, n_iter->neighbor, path, visited, result_vec, g);	
		}

		std::reverse(result_vec.begin(), result_vec.end());
		for (vertex_vec_it v_iter2 = result_vec.begin(); v_iter2 != result_vec.end(); v_iter2++) {
			result_vec_total.push_back(*v_iter2);
		}
		result_vec.clear();
	} 
	return result_vec_total; 	
}

void SetSelection(Agraph_t *agraph, const std::vector<vertex_t>& vertices) {
	std::size_t i = 1;
	Agnode_t *node;
	char ch1[6] = "";
	char num[3] = "";

	for (std::vector<vertex_t>::const_iterator v_it = vertices.begin(); v_it != vertices.end(); v_it++) {
		char label[53] = "";
		sprintf(ch1,"%5d", *v_it);
		node = agnode(agraph, ch1);
		sprintf(num, "%2d ", i);
		strcat(label, num);
		strcat(label, agget(node, "label"));
		agset(node, "label", label);
		agset(node, "color", "red");
		i++;
	}
}

void SetWeight(Agraph_t *agraph, const std::vector<vertex_t>& vertices, float weigth) {
	std::size_t i = 1;
	Agedge_t *edge;
	char ch1[6] = "";
	char ch2[6] = "";
	char weight_ch[9] = "";
	sprintf(weight_ch, "%8.2f", weigth);

	for (std::vector<vertex_t>::const_iterator v_it = vertices.begin(); v_it != vertices.end(); v_it++) {
		sprintf(ch1,"%5d", *v_it);
		if (i > 1) {
			edge = agedge(agraph, agnode(agraph, ch2), agnode(agraph, ch1));
			agset(edge, "weight", weight_ch);
		}
		sprintf(ch2,"%5d", *v_it);
		i++;
	}
}

Agraph_t* Content::LayoutAlgo(const std::set<LayoutEdge, ltedge>& edges,  const std::vector<vertex_t>& add_vertices, const Graph& g) const {
	Agraph_t *agraph;
	Agnode_t* n[g.num_vertices()];
	Agedge_t* e;

    static GVC_t *gvc;

    if (!gvc) gvc = gvContext();

	char ch1[4], ch2[4];
	sprintf(ch1,"%3.1f", 0.8);
	sprintf(ch2,"%3.1f", 0.5);
	agraph = agopen("G", AGRAPHSTRICT);
	agnodeattr(agraph, "width", ch1);
	agnodeattr(agraph, "height", ch2);
	agnodeattr(agraph, "fixedsize", "true");
	sprintf(ch1,"%3d", wxGetApp().fontSize);
	agnodeattr(agraph, "fontsize", ch1);
	agnodeattr(agraph, "color", "green");
	agraphattr(agraph, "splines", "true");
	agedgeattr(agraph, "weight", "0");

	char ch[6];
	char label[50];
	std::set<vertex_t> done;

	for (std::set<LayoutEdge>::iterator e_it = edges.begin(); e_it != edges.end(); e_it++) {
    	if (done.find(e_it->tail) == done.end()) {
			done.insert(e_it->tail);
			sprintf(ch,"%5d", e_it->tail);
			n[e_it->tail] = agnode(agraph, ch);
			(void) strcpy(label, g.vertex_name(e_it->tail).c_str());
			agset(n[e_it->tail], "label", label);
		}

    	if (done.find(e_it->head) == done.end()) {
			done.insert(e_it->head);
			sprintf(ch,"%5d", e_it->head);
			n[e_it->head] = agnode(agraph, ch);		
			(void) strcpy(label, g.vertex_name(e_it->head).c_str());
			agset(n[e_it->head], "label", label);
		}
		e = agedge(agraph, n[e_it->tail], n[e_it->head]);
		sprintf(ch,"%5.3f", e_it->weight);
		agset(e, "weight", ch);
	}

	SetSelection(agraph, add_vertices);
	SetWeight(agraph, add_vertices, 10000);

	for (std::map<std::vector<vertex_t>, PathParam>::const_iterator m_it = paths.begin(); m_it != paths.end(); m_it++)
		SetWeight(agraph, m_it->first, 10000);

	gvLayout(gvc, agraph, "dot");
	return agraph;
}

void GetEdges(std::set<LayoutEdge, ltedge> *edges, const std::vector<vertex_t>& vertices, const Graph& g) {
	std::set<vertex_t> done;
	for (std::vector<vertex_t>::const_iterator v_it = vertices.begin(); v_it != vertices.end(); v_it++) {
    	done.insert(*v_it);
		for (Graph::neighbor_it n_it = g.neighbors_begin(*v_it); n_it != g.neighbors_end(*v_it); n_it++) {
			if (done.find(n_it->neighbor) == done.end()) {
				edges->insert(LayoutEdge(*v_it, n_it->neighbor, exp(-n_it->weight) * 10));
				debug << g.vertex_name(*v_it) << "->" << g.vertex_name(n_it->neighbor) << std::endl;
			}
		}
	}
	debug << std::endl;
}

Agraph_t* Content::ComputeLayout(Graph& g, const std::vector<vertex_t> add_vertices, const PathParam add_params) const {
	std::set<LayoutEdge, ltedge> edges;
	std::vector<vertex_t> vertices;

	for (std::map<std::vector<vertex_t>, PathParam>::const_iterator m_it = paths.begin(); m_it != paths.end(); m_it++) {
		PathParam params = m_it->second;
		if ((params.close_dist == 0) && (params.far_dist == 0)) vertices = (m_it->first);
		else vertices = GetEnvironment(m_it->first, m_it->second, g);
		Graph g_part = g.induced_subgraph(vertices);
		GetEdges(&edges, vertices, g_part);
	}

	if ((add_params.close_dist == 0) && (add_params.far_dist == 0)) vertices = add_vertices;
	else vertices = GetEnvironment(add_vertices, add_params, g);
	Graph g_part = g.induced_subgraph(vertices);
	GetEdges(&edges, vertices, g_part);

	Agraph_t *agraph = LayoutAlgo(edges, add_vertices, g);
	return agraph;
}
