#include <math.h>

#include "evthandler.h"
#include "guiApp.h"
#include "../types.h"

double getDist(double x1, double y1, double x2, double y2) {
	return (sqrt(pow(x1 - x2, 2) + pow(y1 - y2, 2)));
}

CanvasEvtHandler::CanvasEvtHandler(wxShapeEvtHandler *prev, wxShape *shape, 
		const vertex_t& n_vertex, const Graph *n_g):wxShapeEvtHandler(prev, shape) {
	vertex = n_vertex;
	g = n_g;
}

void CanvasEvtHandler::OnLeftClick(double WXUNUSED(x), double WXUNUSED(y), int WXUNUSED(keys), int WXUNUSED(attachment) = 0) {
	wxString msg1;	
	wxString msg2;
	bool modIsSPD =  ((GraphDiagram *)(this->GetShape()->GetCanvas()->GetDiagram()))->modIsSPD;

	wxShape *shape = (wxShape *)this->GetShape();
	if (!shape->IsKindOf(CLASSINFO(wxLineShape))) {
		msg1 << wxT("Vertex:\n") << wxString(g->vertex_name(vertex).c_str(), wxConvUTF8)
			<< wxT("\nDegree: ") << g->deg(vertex);
		msg2 << wxT("Vertex: ") << wxString(g->vertex_name(vertex).c_str(), wxConvUTF8);
	} else {
		wxShape *head = ((wxLineShape *)shape)->GetTo(); 
		wxShape *tail = ((wxLineShape *)shape)->GetFrom();
		vertex_t headvertex = ((CanvasEvtHandler*)(head->GetEventHandler()))->vertex;
		vertex_t tailvertex = ((CanvasEvtHandler*)(tail->GetEventHandler()))->vertex;
		double prob = modIsSPD ? exp(-g->edge_weight(headvertex, tailvertex)) : g->edge_weight(headvertex, tailvertex);
		msg1 << wxT("Edge:\n") << wxString(g->vertex_name(tailvertex).c_str(), wxConvUTF8)
			<< wxT(" -- ") << wxString(g->vertex_name(headvertex).c_str(), wxConvUTF8)
			<< (modIsSPD ? wxT("\nProbability: ") : wxT("\nWeight: ")) << prob;
		msg2 << wxT("Edge: ") << wxString(g->vertex_name(tailvertex).c_str(), wxConvUTF8)
			<< wxT(" -- ") << wxString(g->vertex_name(headvertex).c_str(), wxConvUTF8)
			<< (modIsSPD ? wxT("\tProbability: ") : wxT("\tWeight: ")) << prob;
	}
	wxGetApp().frame->edt_NodeEdgeInfo->SetValue(msg1);
	wxGetApp().frame->StatusBar->SetStatusText(msg2, 0);
}

void CanvasEvtHandler::OnBeginDragLeft(double x, double y, int keys, int attachment = 0) {
	dragDiffX = GetShape()->GetX() - x;
	dragDiffY = GetShape()->GetY() - y;
	wxShapeEvtHandler::OnBeginDragLeft(x, y, keys, attachment);
}

void CanvasEvtHandler::OnEndDragLeft(double x, double y, int keys, int attachment = 0) {	
	GraphDiagram *gdiag = (GraphDiagram *)(this->GetShape()->GetCanvas()->GetDiagram());	

	x += dragDiffX;
	y += dragDiffY;
	gdiag->xPos.erase(gdiag->xPos.find((int)actXPos));
	gdiag->yPos.erase(gdiag->yPos.find((int)actYPos));
	gdiag->xPos.insert((int)x);
	gdiag->yPos.insert((int)y);

	wxList::compatibility_iterator line_it = this->GetShape()->GetLines().GetFirst();
	while(line_it) {
		wxLineShape *line = (wxLineShape *)line_it->GetData();
		wxShape *node2;
		if (line->IsEnd(this->GetShape())) node2 = line->GetFrom();
		else node2 = line->GetTo();

		double node2X = node2->GetX();
		double node2Y = node2->GetY();
		double old_nodeDist = getDist(actXPos, actYPos, node2X, node2Y);
		double new_nodeDist = getDist(x, y, node2X, node2Y);
		double ratio = new_nodeDist / old_nodeDist;
		double old_angle1 = asin((node2Y-actYPos) / old_nodeDist);
		double new_angle1 = asin((node2Y-y) / new_nodeDist);

		wxObjectList::compatibility_iterator cpoint = line->GetLineControlPoints()->GetFirst();
		wxObjectList::compatibility_iterator firstPoint = line->GetLineControlPoints()->GetFirst();
		wxObjectList::compatibility_iterator lastPoint = line->GetLineControlPoints()->GetLast();
		while (cpoint)
		{
			wxRealPoint *coord = (wxRealPoint *)cpoint->GetData();

			if ((cpoint != firstPoint) && (cpoint != lastPoint)) {
				int pointXPos = (int)(coord->x);
				int pointYPos = (int)(coord->y);

				if (gdiag->xPos.find(pointXPos) == gdiag->xPos.end()) pointXPos -= 1;
				if (gdiag->xPos.find(pointXPos) == gdiag->xPos.end()) std::cerr << "pointXPos " 
					<< pointXPos << " is not stored in multiset xPos" << std::endl;
				else gdiag->xPos.erase(gdiag->xPos.find(pointXPos));

				if (gdiag->yPos.find(pointYPos) == gdiag->yPos.end()) pointYPos -= 1;
				if (gdiag->yPos.find(pointYPos) == gdiag->yPos.end()) std::cerr << "pointYPos " 
					<< pointYPos << " is not stored in multiset yPos" << std::endl;
				else gdiag->yPos.erase(gdiag->yPos.find(pointYPos));
			}


			double old_distToNode = getDist(actXPos, actYPos, coord->x, coord->y);
			double old_angle2;

			int sign = ((node2X > actXPos)? -1 : 1);
			if (sign * coord->x <  sign * actXPos) old_angle2 = asin((coord->y-actYPos) / old_distToNode) - old_angle1;
			else old_angle2 = asin(1) * 2 - asin((coord->y-actYPos) / old_distToNode) - old_angle1;
			if (node2X <= actXPos) old_angle2 *= -1;

			double old_distOnDiag = cos(old_angle2) * old_distToNode;
			double new_distOnDiag = old_distOnDiag * ratio;
			double distToDiag = sin(old_angle2) * old_distToNode;
			double new_distToNode = sqrt(pow(new_distOnDiag, 2) + pow(distToDiag, 2));
			double new_angle2 = asin(((distToDiag >= 0)? 1 : 1) * distToDiag / new_distToNode);

			coord->x = ((x > node2X)? -1 : 1) * cos(((x > node2X)? -1 : 1) * new_angle2 + new_angle1) * new_distToNode + x;
			coord->y = ((y > node2Y)? 1 : 1) * sin(((x > node2X)? -1 : 1) * new_angle2 + new_angle1) * new_distToNode + y;
			if ((cpoint != firstPoint) && (cpoint != lastPoint)) {
				gdiag->xPos.insert((int)(coord->x));
				gdiag->yPos.insert((int)(coord->y));
			}
			cpoint = cpoint->GetNext();
		}
		line_it = line_it->GetNext();
	}

	wxShapeEvtHandler::OnEndDragLeft(x - dragDiffX, y - dragDiffY, keys, attachment);

	actXPos = x;
	actYPos = y;
	gdiag->g_width = *(gdiag->xPos.rbegin()) - *(gdiag->xPos.begin()) + gdiag->nodeWidth; 
	gdiag->g_height = *(gdiag->yPos.rbegin()) - *(gdiag->yPos.begin()) + gdiag->nodeHeight; 
	this->GetShape()->GetCanvas()->Refresh();
}
