#include <vector>
#include <stdio.h>
#include <math.h>
#include <iostream>

#include <wx/dcps.h>
#include <wx/image.h>
#include <gvc.h>

#include "graphcanvas.h"
#include "guiApp.h"
#include "evthandler.h"
#include "../types.h"

#define SCROLLRATE_X 2
#define SCROLLRATE_Y 2

BEGIN_EVENT_TABLE(GraphCanvas,wxShapeCanvas)
	EVT_SIZE(GraphCanvas::OnSize)
END_EVENT_TABLE()


GraphCanvas::GraphCanvas(wxWindow *parent = NULL, wxWindowID id = -1, 
				const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, 
				long style = wxBORDER) : wxShapeCanvas(parent, id, pos, size, style) {
	ExportFileDialog = new wxFileDialog(this, wxT("Export graph to file"), wxT(""), wxT(""), 
		wxT("PostScript (*.ps)|*.ps|Portable Network Graphics (*.png)|*.png|DOT Language (*.dot)|*.dot"), wxSAVE);
}

GraphCanvas* GraphCanvas::CreateCanvas(wxView *WXUNUSED(view), wxWindow *parent) {
	int width, height;
	parent->GetClientSize(&width, &height);

	GraphCanvas *Canvas = new GraphCanvas(parent, wxID_ANY, wxPoint(0, 0), wxSize(width, height), wxNO_BORDER);
	Canvas->SetCursor(wxCursor(wxCURSOR_HAND));
	Canvas->SetBackgroundColour(wxColour(255, 255, 255));
	return Canvas;
}

void GraphCanvas::DrawGraph(Agraph_t *agraph, Graph *g) {
	std::size_t num_vertices = g->num_vertices();
	GraphDiagram *gdiag = (GraphDiagram *)GetDiagram();

	int tmp1, tmp2, can_width, can_height;
	float can_center_x, can_center_y, g_center_x, g_center_y;
	sscanf(agget(agraph, "bb"), "%d %d %f %f", &tmp1, &tmp2, &(gdiag->g_width), &(gdiag->g_height));
	gdiag->g_width = gdiag->g_width * gdiag->g_scale;
	gdiag->g_height = gdiag->g_height * gdiag->g_scale;

	SetScrollbars(SCROLLRATE_X, SCROLLRATE_Y,  
		(int)(gdiag->g_width / SCROLLRATE_X) + 1, (int)(gdiag->g_height / SCROLLRATE_Y) + 1);
	GetSize(&can_width, &can_height);
	gdiag->can_width = (float)can_width;
	gdiag->can_height = (float)can_height;

	if (gdiag->can_width < gdiag->g_width) gdiag->can_width = gdiag->g_width;
	if (gdiag->can_height < gdiag->g_height) gdiag->can_height = gdiag->g_height;
	g_center_x = gdiag->g_width / 2;
	g_center_y = gdiag->g_height / 2;
	can_center_x = gdiag->can_width / 2;
	can_center_y = gdiag->can_height / 2;
	if (can_center_x < g_center_x) can_center_x = g_center_x;
	if (can_center_y < g_center_y) can_center_y = g_center_y;

	wxShape* Shapes[num_vertices];

	wxClientDC dc(this);
	int fontSize = GetFontSize(dc, .9 * .8 * 72 * gdiag->g_scale, .9 * .5 * 72 * gdiag->g_scale);
    
	for (Agnode_t* node = agfstnode(agraph); node; node = agnxtnode(agraph, node)) {
	int i = atoi(node->name);
		if (g->deg(i) == 0) continue;

		Shapes[i] = new wxEllipseShape(.8 * 72 * gdiag->g_scale, .5 * 72 * gdiag->g_scale);
		Shapes[i]->AssignNewIds();
	    Shapes[i]->SetEventHandler(new CanvasEvtHandler(Shapes[i], Shapes[i], i, g));
	    Shapes[i]->SetCentreResize(false);
	    Shapes[i]->SetPen(wxBLACK_PEN);
		if (strcmp(agget(node, "color"), "red") == 0) {
			Shapes[i]->SetBrush(new wxBrush(wxGetApp().colorSelV));
		} else {
			Shapes[i]->SetBrush(new wxBrush(wxGetApp().colorDeSelV));
		}
		Shapes[i]->SetFont(new wxFont(fontSize, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL));
	    wxString vertexLabel = wxString(agget(node, "label"), wxConvUTF8);
		Shapes[i]->AddText(vertexLabel);
		((CanvasEvtHandler *)(Shapes[i]->GetEventHandler()))->label = vertexLabel;

		Shapes[i]->SetX(can_center_x - g_center_x + ND_coord_i(node).x * gdiag->g_scale);
		Shapes[i]->SetY(can_center_y + g_center_y - ND_coord_i(node).y * gdiag->g_scale);
		gdiag->xPos.insert((int)(Shapes[i]->GetX()));
		gdiag->yPos.insert((int)(Shapes[i]->GetY()));
		((CanvasEvtHandler *)(Shapes[i]->GetEventHandler()))->actXPos = Shapes[i]->GetX();
		((CanvasEvtHandler *)(Shapes[i]->GetEventHandler()))->actYPos = Shapes[i]->GetY();

		GetDiagram()->AddShape(Shapes[i]);
		Shapes[i]->Show(true);
	}

	wxLineShape* newLine = new wxLineShape;
	for (Agnode_t* node = agfstnode(agraph); node; node = agnxtnode(agraph, node)) {
		for (Agedge_t* edge = agfstout(agraph, node); edge; edge = agnxtout(agraph, edge)) {
			vertex_t head = atoi((edge->head)->name);
			vertex_t tail = atoi((edge->tail)->name);

			double prob;
			if (gdiag->modIsSPD) prob = exp(-g->edge_weight(head, tail));
			else prob = 1 - ((g->edge_weight(head, tail) - gdiag->weightMin) / gdiag->weightDiff);
			wxPen *line_pen = new wxPen();
			line_pen->SetColour(GetLineColor(prob));
		    line_pen->SetWidth(wxGetApp().lineWidth);

			newLine = new wxLineShape();
			newLine->AssignNewIds();
		    newLine->SetEventHandler(new CanvasEvtHandler(newLine, newLine, 0, g));
			newLine->SetPen(line_pen);
		    newLine->SetBrush(wxBLUE_BRUSH);
		    newLine->MakeLineControlPoints(ED_spl(edge)->list->size - 1);
		    newLine->SetSpline(true);

			Shapes[tail]->AddLine(newLine, Shapes[head]);
			GetDiagram()->AddShape(newLine);
			newLine->Show(true);

			int cpoint_count = 0;
			wxObjectList::compatibility_iterator cpoint = newLine->GetLineControlPoints()->GetFirst();
			wxObjectList::compatibility_iterator firstPoint = newLine->GetLineControlPoints()->GetFirst();
			wxObjectList::compatibility_iterator lastPoint = newLine->GetLineControlPoints()->GetLast();

			while (cpoint) {
				wxRealPoint *coord = (wxRealPoint *)cpoint->GetData();
				coord->x = can_center_x - g_center_x + ((ED_spl(edge)->list->list[cpoint_count]).x + 
					(ED_spl(edge)->list->list[cpoint_count+1]).x) / 2 * gdiag->g_scale;
				coord->y = can_center_y + g_center_y - ((ED_spl(edge)->list->list[cpoint_count]).y + 
					(ED_spl(edge)->list->list[cpoint_count+1]).y) / 2 * gdiag->g_scale;
				if ((cpoint != firstPoint) && (cpoint != lastPoint)) {
					gdiag->xPos.insert((int)(coord->x));
					gdiag->yPos.insert((int)(coord->y));
				}
				cpoint = cpoint->GetNext();
				cpoint_count++;
			}

	     	Shapes[tail]->Move(dc, Shapes[tail]->GetX(), Shapes[tail]->GetY());
     		Shapes[head]->Move(dc, Shapes[head]->GetX(), Shapes[head]->GetY());
		}
	}
    this->Redraw(dc);
	this->Refresh();

	gdiag->nodeWidth = .8 * 72 * gdiag->g_scale;
	gdiag->nodeHeight = .5 * 72 * gdiag->g_scale;
}

wxColour GraphCanvas::GetLineColor(double prob) {
	unsigned char red = 0, green = 255, blue = 0;
	if (prob >= 0.5) green = 255 - (unsigned char)((prob - 0.5) * 350);
	else {
		red = 200 - (unsigned char)(prob * 400);
		blue = 200 - (unsigned char)(prob * 400);
	}
	return wxColour(red, green, blue);
}

void GraphCanvas::ZoomFit() {
	GraphDiagram *gdiag = (GraphDiagram *)GetDiagram();

	int canWidth, canHeight;
	GetClientSize(&canWidth, &canHeight);

	double scaleX = gdiag->g_scale * (canWidth) / gdiag->g_width;
	double scaleY = gdiag->g_scale * (canHeight) / gdiag->g_height;
	double centerX = (*(gdiag->xPos.begin()) + *(gdiag->xPos.rbegin())) / 2;
	double centerY = (*(gdiag->yPos.begin()) + *(gdiag->yPos.rbegin())) / 2;

	SetScrollbars(SCROLLRATE_X, SCROLLRATE_Y,  
		(int)(((*(gdiag->xPos.rbegin()) + gdiag->nodeWidth / 2) * 2) / SCROLLRATE_X) + 1, 
		(int)(((*(gdiag->yPos.rbegin()) + gdiag->nodeHeight / 2) * 2) / SCROLLRATE_Y) + 1, 
		(int)((centerX - canWidth / 2) / SCROLLRATE_X), (int)((centerY - canHeight / 2) / SCROLLRATE_Y));

	Zoom(std::min(scaleX, scaleY) - 0.035);

	wxString scale;
	wxString format = wxT("%6.2f");
	scale.Printf(format.c_str(), (gdiag->g_scale * 100));
	scale << wxT("%");
	wxGetApp().frame->cbx_Zoom->SetValue(scale);
}

void GraphCanvas::Zoom(double scale) {
	GraphDiagram *gdiag = (GraphDiagram *)GetDiagram();

	double new_scale = scale / gdiag->g_scale;
	gdiag->g_scale = scale;
	scale = new_scale;

	if (gdiag->GetCount() == 0) return;

	int startX, startY, canWidth, canHeight;
	GetSize(&canWidth, &canHeight);
	GetViewStart(&startX, &startY);

	float centerX = canWidth / 2 + startX * SCROLLRATE_X;
	float centerY = canHeight / 2 + startY * SCROLLRATE_Y;

	gdiag->g_width = gdiag->g_width * scale;
	gdiag->g_height = gdiag->g_height * scale;
	gdiag->can_width = std::max(canWidth, (int)gdiag->g_width + 1);
	gdiag->can_height = std::max(canHeight, (int)gdiag->g_height + 1);

	float distX_before = centerX - (*(gdiag->xPos.begin()) - gdiag->nodeWidth / 2);
	float distY_before = centerY - (*(gdiag->yPos.begin()) - gdiag->nodeHeight / 2);
	float distX_after = distX_before * scale;
	float distY_after = distY_before * scale;
	float distX_before2 = centerX - *(gdiag->xPos.begin());
	float distY_before2 = centerY - *(gdiag->yPos.begin());
	float offsetX = -(*(gdiag->xPos.begin()) - gdiag->nodeWidth / 2) * scale;
	float offsetY = -(*(gdiag->yPos.begin()) - gdiag->nodeHeight / 2) *scale;

	if (distX_after < canWidth / 2) offsetX += canWidth / 2 - distX_after;
	if (distY_after < canHeight / 2) offsetY += canHeight / 2 - distY_after;

	RepositionShapes(offsetX, offsetY, canWidth, canHeight, scale);

	float new_centerX = distX_before2 * scale + *(gdiag->xPos.begin());
	float new_centerY = distY_before2 * scale + *(gdiag->yPos.begin());

	SetScrollbars(SCROLLRATE_X, SCROLLRATE_Y,  
		(int)(std::max((float)(*(gdiag->xPos.rbegin()) + gdiag->nodeWidth / 2), new_centerX + canWidth / 2) / SCROLLRATE_X) + 1, 
		(int)(std::max((float)(*(gdiag->yPos.rbegin()) + gdiag->nodeHeight / 2), new_centerY + canHeight / 2) / SCROLLRATE_Y) + 1, 
		(int)((new_centerX - canWidth / 2) / SCROLLRATE_X), (int)((new_centerY - canHeight / 2) / SCROLLRATE_Y), true);

	this->Refresh();
}

void GraphCanvas::OnSize(wxSizeEvent& event) {
	int startX, startY;
	GetViewStart(&startX, &startY);

	GraphDiagram *gdiag = (GraphDiagram *)GetDiagram();
	if (gdiag->GetCount() == 0) return;

	wxSize size = event.GetSize();
	int newCanWidth = size.GetWidth();
	int newCanHeight = size.GetHeight();

	newCanWidth = std::max(newCanWidth, (int)gdiag->g_width + 1);
	newCanHeight = std::max(newCanHeight, (int)gdiag->g_height + 1);
	float diffX = std::max((newCanWidth - gdiag->can_width) / 2, -(float)*(gdiag->xPos.begin()) + gdiag->nodeWidth / 2);
	float diffY = std::max((newCanHeight - gdiag->can_height) / 2, -(float)*(gdiag->yPos.begin()) + gdiag->nodeHeight / 2);
	gdiag->can_width = newCanWidth;
	gdiag->can_height = newCanHeight;

	RepositionShapes(diffX, diffY, newCanWidth, newCanHeight);

	SetScrollbars(SCROLLRATE_X, SCROLLRATE_Y,  
		(int)(std::max((*(gdiag->xPos.rbegin()) + gdiag->nodeWidth / 2), (float)(startX + size.GetWidth())) / SCROLLRATE_X) + 1, 
		(int)(std::max((*(gdiag->yPos.rbegin()) + gdiag->nodeHeight / 2), (float)(startY + size.GetHeight())) / SCROLLRATE_Y) + 1, 
		startX , startY, true);

	this->Refresh();
	event.Skip();
}

void GraphCanvas::RepositionShapes(float offsetX, float offsetY, float WXUNUSED(canWidth), float WXUNUSED(canHeight), float scale) {
	GraphDiagram *gdiag = (GraphDiagram *)GetDiagram();

	double width, height;
	wxFont* font;	
	wxClientDC dc(this);

	gdiag->xPos.clear();
	gdiag->yPos.clear();
	wxObjectList::compatibility_iterator node = gdiag->GetShapeList()->GetFirst();
	while (node) {
		wxShape *eachShape = (wxShape *)node->GetData();
		if (!eachShape->IsKindOf(CLASSINFO(wxLineShape))) {	
//			int shapeXPos = (int)(((CanvasEvtHandler *)(eachShape->GetEventHandler()))->actXPos);
//			int shapeYPos = (int)(((CanvasEvtHandler *)(eachShape->GetEventHandler()))->actYPos);

//			if (gdiag->xPos.find(shapeXPos) == gdiag->xPos.end()) shapeXPos -= 1;
//			if (gdiag->xPos.find(shapeXPos) == gdiag->xPos.end()) std::cerr << "ShapeXPos " 
//				<< shapeXPos << " is not stored in multiset xPos" << std::endl;
//			else gdiag->xPos.erase(gdiag->xPos.find(shapeXPos));

//			if (gdiag->yPos.find(shapeYPos) == gdiag->yPos.end()) shapeYPos -= 1;
//			if (gdiag->yPos.find(shapeYPos) == gdiag->yPos.end()) std::cerr << "ShapeYPos " 
//				<< shapeYPos << " is not stored in multiset yPos" << std::endl;
//			else gdiag->yPos.erase(gdiag->yPos.find(shapeYPos));

			eachShape->SetX(eachShape->GetX() * scale + offsetX);
			eachShape->SetY(eachShape->GetY() * scale + offsetY);

			if (scale != 1) {
				eachShape->GetBoundingBoxMax(&width, &height);
				eachShape->SetSize(width * scale, height * scale);

				int fontSize = GetFontSize(dc, .9 * width * scale, .9 * height * scale);		
				font = eachShape->GetFont();
				font->SetPointSize(fontSize);
				eachShape->ClearText();
				eachShape->AddText(((CanvasEvtHandler *)(eachShape->GetEventHandler()))->label);
			}
			gdiag->xPos.insert((int)(eachShape->GetX()));
			gdiag->yPos.insert((int)(eachShape->GetY()));
			((CanvasEvtHandler *)(eachShape->GetEventHandler()))->actXPos = eachShape->GetX();
			((CanvasEvtHandler *)(eachShape->GetEventHandler()))->actYPos = eachShape->GetY();

		} else {

			wxObjectList::compatibility_iterator cpoint = ((wxLineShape *)eachShape)->GetLineControlPoints()->GetFirst();
			wxObjectList::compatibility_iterator firstPoint = ((wxLineShape *)eachShape)->GetLineControlPoints()->GetFirst();
			wxObjectList::compatibility_iterator lastPoint = ((wxLineShape *)eachShape)->GetLineControlPoints()->GetLast();
			while (cpoint)
			{
				wxRealPoint *coord = (wxRealPoint *)cpoint->GetData();
//				if ((cpoint != firstPoint) && (cpoint != lastPoint)) {
//					int pointXPos = (int)(coord->x);
//					int pointYPos = (int)(coord->y);

//					if (gdiag->xPos.find(pointXPos) == gdiag->xPos.end()) pointXPos -= 1;
//					if (gdiag->xPos.find(pointXPos) == gdiag->xPos.end()) std::cerr << "pointXPos " 
//						<< pointXPos << " is not stored in multiset xPos" << std::endl;
//					else gdiag->xPos.erase(gdiag->xPos.find(pointXPos));

//					if (gdiag->yPos.find(pointYPos) == gdiag->yPos.end()) pointYPos -= 1;
//					if (gdiag->yPos.find(pointYPos) == gdiag->yPos.end()) std::cerr << "pointYPos " 
//						<< pointYPos << " is not stored in multiset yPos" << std::endl;
//					else gdiag->yPos.erase(gdiag->yPos.find(pointYPos));
//				}
				coord->x = coord->x * scale + offsetX;
				coord->y = coord->y * scale + offsetY;
				if ((cpoint != firstPoint) && (cpoint != lastPoint)) {
					gdiag->xPos.insert((int)(coord->x));
					gdiag->yPos.insert((int)(coord->y));
				}
				cpoint = cpoint->GetNext();
			}
		}
		node = node->GetNext();
	}

//	node = this->GetDiagram()->GetShapeList()->GetFirst();
//	while (node)
//	{
//		wxShape *eachShape = (wxShape *)node->GetData();
//		eachShape->Move(dc, eachShape->GetX(), eachShape->GetY());
//		node = node->GetNext();
//	}

	if (scale != 1) {
		gdiag->nodeWidth = width * scale;
		gdiag->nodeHeight = height * scale;
	}
}

int GraphCanvas::GetFontSize(wxDC& WXUNUSED(dc), float WXUNUSED(spaceWidth), float WXUNUSED(spaceHeight)) {
	GraphDiagram *gdiag = (GraphDiagram *)GetDiagram();
	return ((int) (gdiag->g_scale * wxGetApp().fontSize));

//	wxString testString = wxT("XXXXXXXXX");
//	int width, height, i;
//	for (i = 1; i < 30; i++) {
//		dc.GetTextExtent(testString, &width, &height, NULL, NULL, 
//			new wxFont(i, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL));
//		if ((width > spaceWidth) || (height > spaceHeight)) break;
//	}
//	return (i - 1); 
}

void GraphCanvas::PrintGraph() {
	wxMessageDialog(this, wxT("Printing is not supported yet!")).ShowModal();
}

void GraphCanvas::ExportGraph() {
	wxClientDC canvas_dc(this);
	GraphDiagram *gdiag = (GraphDiagram *)GetDiagram();

    if (ExportFileDialog->ShowModal() == wxID_OK) 
	{
		wxString filename = ExportFileDialog->GetPath();
		if (ExportFileDialog->GetFilterIndex() == 0 && (filename.Right(3).CmpNoCase(wxT(".ps")) != 0)) filename.Append(wxT(".ps"));
		if (ExportFileDialog->GetFilterIndex() == 1 && (filename.Right(4).CmpNoCase(wxT(".png")) != 0)) filename.Append(wxT(".png"));
		if (ExportFileDialog->GetFilterIndex() == 2 && (filename.Right(4).CmpNoCase(wxT(".dot")) != 0)) filename.Append(wxT(".dot"));
		if (ExportFileDialog->GetFilterIndex() == 3 && (filename.Right(4).CmpNoCase(wxT(".ogl")) != 0)) filename.Append(wxT(".ogl"));
	
		if (ExportFileDialog->GetFilterIndex() == 1) {
			wxImage::AddHandler(new wxPNGHandler);
			wxBitmap bitmap(*(gdiag->xPos.rbegin()) + 400, *(gdiag->yPos.rbegin()) + 400);

			wxMemoryDC tempDC;
			tempDC.SelectObject(bitmap);
			tempDC.SetBrush(*wxWHITE_BRUSH);
			tempDC.SetPen(*wxWHITE_PEN);
			tempDC.DrawRectangle(0, 0, *(gdiag->xPos.rbegin()) + 400, *(gdiag->yPos.rbegin()) + 400);
			tempDC.ResetBoundingBox();

			Redraw(tempDC);
			std::size_t lineWidth = wxGetApp().lineWidth;
			std::size_t graphWidth = tempDC.MaxX() - tempDC.MinX() + lineWidth + 3;
			std::size_t graphHeight = tempDC.MaxY() - tempDC.MinY() + lineWidth + 3;

			tempDC.Blit(-1, -1, graphWidth, graphHeight, &tempDC, tempDC.MinX() - (int)(lineWidth / 2) - 1, tempDC.MinY() - (int)(lineWidth / 2));
			tempDC.SelectObject(wxNullBitmap);

			bitmap.SetWidth(graphWidth - 1);
			bitmap.SetHeight(graphHeight - 1);
			bitmap.SaveFile(filename, wxBITMAP_TYPE_PNG);
		}
		else if (ExportFileDialog->GetFilterIndex() == 2) {
			wxCopyFile(wxT("./currentgraph.dot"), filename, true);
		}
		else if (ExportFileDialog->GetFilterIndex() == 3) {
//			((GraphDiagram *)GetDiagram())->SaveFile(filename);
		}
		else {
#ifndef __WXMSW__
			wxPrintData g_print_data;
			g_print_data.SetFilename(filename);
			g_print_data.SetPrintMode(wxPRINT_MODE_FILE);
			g_print_data.SetPaperId(wxPAPER_A4);
			wxPostScriptDC ps_dc(g_print_data);
			ps_dc.StartDoc(wxT("Printing PS"));

			float scaleX = 475 / (*(gdiag->xPos.rbegin()) - *(gdiag->xPos.begin()) + gdiag->nodeWidth);
			float scaleY = 720 / (*(gdiag->yPos.rbegin()) - *(gdiag->yPos.begin()) + gdiag->nodeHeight);
			float scale = std::min((float)1.0, std::min(scaleX, scaleY));
			ps_dc.SetUserScale(scale, scale);
			ps_dc.SetDeviceOrigin((int)(-(*(gdiag->xPos.begin()) - gdiag->nodeWidth / 2) * scale + 60 + (1 - (scale / scaleX)) * 237), 
				(int)(-(*(gdiag->yPos.begin()) - gdiag->nodeHeight / 2) * scale + 60));

			wxFont *font;
			int old_fontsize = GetFontSize(canvas_dc, gdiag->nodeWidth * 0.9, gdiag->nodeHeight * 0.9);
			int fontsize = (int)(GetFontSize(ps_dc, gdiag->nodeWidth * 0.8, gdiag->nodeHeight * 0.8) * 1.5);
			wxObjectList::compatibility_iterator node = this->GetDiagram()->GetShapeList()->GetFirst();
			while (node) {
				wxShape *eachShape = (wxShape *)node->GetData();
				if (!eachShape->IsKindOf(CLASSINFO(wxLineShape))) {
					font = eachShape->GetFont();
					font->SetPointSize(fontsize);
					eachShape->ClearText();
					eachShape->AddText(((CanvasEvtHandler *)(eachShape->GetEventHandler()))->label);
				}
				eachShape->Move(ps_dc, eachShape->GetX(), eachShape->GetY());
				if (!eachShape->IsKindOf(CLASSINFO(wxLineShape))) {
					font->SetPointSize(old_fontsize);
					eachShape->ClearText();
					eachShape->AddText(((CanvasEvtHandler *)(eachShape->GetEventHandler()))->label);
				} 
				node = node->GetNext();
			}
			ps_dc.EndDoc();
#else
	wxMessageBox(wxT("Unfortunately, the PostScript format is currently not supported under Windows."), wxT("Error"), wxICON_WARNING);
#endif
		}
	}
}
